data segment para
	io8254_0	equ		230h	; 8254定时器0的控制端口地址
	io8254_1	equ		231h	; 8254定时器1的控制端口地址
	io8254_2	equ		232h	; 8254定时器2的控制端口地址
	io8254_k	equ		233h	; 8254定时器的控制字端口地址
	io8255_a	equ		210h	; 8255的A端口地址
	io8255_b	equ		211h	; 8255的B端口地址
	io8255_c	equ		212h	; 8255的C端口地址
	io8255_ctl	equ		213h	; 8255的控制字端口地址
	ad_io		equ		220h	; ADC0809的基地址
	data1		db		0		; 用于存储ADC转换结果的变量
data ends

; 堆栈段定义
ss_seg segment stack
	dw	100	dup(0)	; 定义100个双字（即200个字节）的堆栈空间
ss_seg ends

; 代码段定义
code segment para
assume cs:code,ds:data,ss:ss_seg

; 主程序开始
main proc far
	; 初始化数据段寄存器
	mov		ax,		data
	mov		ds,		ax
	nop					

	; 配置8255为模式0，A、B、C三组均为输出
	mov		dx,		io8255_ctl
	mov		al,		10010011b
	out		dx,		al

	; 配置8254定时器0的工作模式
	mov		dx,		io8254_k
	mov		al,		36h
	out		dx,		al
	; 设置定时器0的初值（低8位和高8位）
	mov		ax,		100
	mov		dx,		io8254_0
	out		dx,		al		; 低8位
	mov		al,		ah
	out		dx,		al		; 高8位

	; 配置8254定时器1的工作模式
	mov		dx,		io8254_k
	mov		al,		76h
	out		dx,		al
	; 设置定时器1的初值（低8位和高8位）
	mov		ax,		260
	mov		dx,		io8254_1
	out		dx,		al		; 低8位
	mov		al,		ah
	out		dx,		al		; 高8位

; 主循环开始
lop:
	; 配置8255的控制字，准备进行ADC转换
	mov		dx,		io8255_ctl
	mov		al,		91h		; 设置8255的工作模式
	out		dx,		al
	; 初始化8255的B端口为0
	mov		dx,		io8255_b
	mov		al,		0
	out		dx,		al

; 等待ADC转换完成
loop1:
	; 启动ADC转换
	mov		dx,		ad_io
	out		dx,		al
	; 指向ADC的状态端口
	mov		dx,		ad_io
	add		dx,		2
loop2:
	; 读取ADC的状态
	in		al,		dx
	; 检查EOC（转换结束标志）是否为1
	test	al,		01h
	jz		loop2		; 如果EOC为0，继续等待

	; 读取ADC的转换结果
	mov		dx,		ad_io
	inc		dx
	in		al,		dx		; 结果存入AL寄存器

	; 将转换结果存储到data1变量中
	mov		data1,	al
	; 将转换结果输出到8255的A端口
	mov		dx,		io8255_a
	out		dx,		al
	; 确保AL的值不为0
	and		al,		0ffh
	mov		dx,		io8255_b	
	out		dx,		al
	; 如果AL为0，则将其加1
	jnz		lop3
	inc		al
lop3:
	; 将转换结果保存到BL寄存器中
	mov		bl,		al

; 等待外部信号
lop4:
	mov		dx,		io8255_c
	in		al,		dx
	; 检查C端口的最低位是否为0
	and		al,		01h
	jnz		lop4		; 如果不为0，继续等待

	; 配置8255的控制字，改变某些端口的状态
	mov		dx,		io8255_ctl
	mov		al,		0fh		; 设置PC7
	out		dx,		al

	; 配置8254定时器2的工作模式
	mov		dx,		io8254_k
	mov		al,		0b0h
	out		dx,		al

	; 将BL寄存器的值（即ADC的转换结果）写入定时器2的初值寄存器
	xor		ax,		ax		; 清零AX寄存器
	mov		al,		bl
	mov		dx,		io8254_2
	out		dx,		al		; 低8位
	mov		al,		ah
	out		dx,		al		; 高8位

lop1:
	mov		dx,		io8255_c
	in		al,		dx
	; 检查C端口的第1位是否为1
	and		al,		02h
	jz		lop1		; 如果为0，继续等待

	; 配置8255的控制字，再次改变端口状态
	mov		dx,		io8255_ctl
	mov		al,		0eh
	out		dx,		al

; 等待C端口的最低位变为0
lop2:
	mov		dx,		io8255_c
	in		al,		dx
	and		al,		01h
	jz		lop2

	; 跳转到loop1，进行下一次ADC转换
	jmp		loop1

	; 程序结束，返回DOS
	mov		ax,		4c00h
	int		21h
main endp
code ends
end main